# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacPolicyAssociationTests(rbac_base.IdentityV3RbacBaseTests,
                                           metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacPolicyAssociationTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.policies_client
        cls.admin_client = cls.os_system_admin
        cls.admin_policies_client = cls.admin_client.policies_client

    @classmethod
    def resource_setup(cls):
        super(IdentityV3RbacPolicyAssociationTests, cls).resource_setup()
        cls.policy_id = cls.admin_policies_client.create_policy(
            blob=data_utils.rand_uuid_hex(),
            type=data_utils.rand_uuid_hex())['policy']['id']
        cls.addClassResourceCleanup(
            cls.admin_policies_client.delete_policy,
            policy_id=cls.policy_id)
        cls.region_id = cls.admin_client.regions_client.create_region(
            region_id=data_utils.rand_name())['region']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.regions_client.delete_region,
            cls.region_id)
        svc_client = cls.admin_client.identity_services_v3_client
        cls.service_id = svc_client.create_service(
            type=data_utils.rand_name())['service']['id']
        cls.addClassResourceCleanup(svc_client.delete_service, cls.service_id)
        cls.endpoint_id = cls.admin_client.endpoints_v3_client.create_endpoint(
            interface='public',
            url='http://localhost/foo',
            service_id=cls.service_id)['endpoint']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.endpoints_v3_client.delete_endpoint,
            endpoint_id=cls.endpoint_id)

    @abc.abstractmethod
    def test_identity_create_policy_association_for_endpoint(self):
        """Test identity:create_policy_association_for_endpoint policy.

        This test must check:
          * whether the persona can associate a policy with an endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_create_policy_association_for_service(self):
        """Test identity:create_policy_association_for_service policy.

        This test must check:
          * whether the persona can associate a policy with a service
        """
        pass

    @abc.abstractmethod
    def test_identity_create_policy_association_for_region_and_service(self):
        """Test identity:create_policy_association_for_region_and_service.

        This test must check:
          * whether the persona can associate a policy with a region and
            service
        """
        pass

    @abc.abstractmethod
    def test_identity_check_policy_association_for_endpoint(self):
        """Test identity:check_policy_association_for_endpoint policy.

        This test must check:
          * whether the persona can check a policy association for an endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_check_policy_association_for_service(self):
        """Test identity:check_policy_association_for_service policy.

        This test must check:
          * whether the persona can check a policy association for a service
        """
        pass

    @abc.abstractmethod
    def test_identity_check_policy_association_for_region_and_service(self):
        """Test identity:check_policy_association_for_region_and_service.

        This test must check:
          * whether the persona can check a policy association for a region and
            service
        """
        pass

    @abc.abstractmethod
    def test_identity_get_policy_for_endpoint(self):
        """Test identity:get_policy_for_endpoint policy.

        This test must check:
          * whether the persona can get a policy for an endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_list_endpoints_for_policy(self):
        """Test identity:list_endpoints_for_policy policy.

        This test must check:
          * whether the persona can list endpoints for a policy
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_policy_association_for_endpoint(self):
        """Test identity:delete_policy_association_for_endpoint policy.

        This test must check
          * whether the persona can delete a policy association for an endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_policy_association_for_service(self):
        """Test identity:delete_policy_association_for_service policy.

        This test must check
          * whether the persona can delete a policy association for a service
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_policy_association_for_region_and_service(self):
        """Test identity:delete_policy_association_for_region_and_service policy.

        This test must check
          * whether the persona can delete a policy association for a region
            and service
        """  # noqa: E501
        pass


class SystemAdminTests(
    IdentityV3RbacPolicyAssociationTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_policy_association_for_endpoint(self):
        self.do_request(
            'update_policy_association_for_endpoint',
            expected_status=204,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_create_policy_association_for_service(self):
        self.do_request(
            'update_policy_association_for_service',
            expected_status=204,
            policy_id=self.policy_id, service_id=self.service_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_service,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_create_policy_association_for_region_and_service(self):
        self.do_request(
            'update_policy_association_for_region_and_service',
            expected_status=204,
            policy_id=self.policy_id, service_id=self.service_id,
            region_id=self.region_id)
        delete_fn = getattr(
            self.admin_policies_client,
            'delete_policy_association_for_region_and_service'
        )
        self.addCleanup(delete_fn,
                        policy_id=self.policy_id,
                        service_id=self.service_id,
                        region_id=self.region_id)

    def test_identity_check_policy_association_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'show_policy_association_for_endpoint',
            expected_status=204,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_check_policy_association_for_service(self):
        self.admin_policies_client.update_policy_association_for_service(
            policy_id=self.policy_id, service_id=self.service_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_service,
            policy_id=self.policy_id, service_id=self.service_id)
        self.do_request(
            'show_policy_association_for_service',
            expected_status=204,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_check_policy_association_for_region_and_service(self):
        update_fn = getattr(
            self.admin_policies_client,
            'update_policy_association_for_region_and_service'
        )
        update_fn(policy_id=self.policy_id,
                  service_id=self.service_id,
                  region_id=self.region_id)
        delete_fn = getattr(
            self.admin_policies_client,
            'delete_policy_association_for_region_and_service'
        )
        self.addCleanup(delete_fn,
                        policy_id=self.policy_id,
                        service_id=self.service_id,
                        region_id=self.region_id)
        self.do_request(
            'show_policy_association_for_region_and_service',
            expected_status=204,
            policy_id=self.policy_id,
            service_id=self.service_id,
            region_id=self.region_id)

    def test_identity_get_policy_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'show_policy_for_endpoint',
            expected_status=200,
            endpoint_id=self.endpoint_id)

    def test_identity_list_endpoints_for_policy(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'list_endpoints_for_policy',
            expected_status=200,
            policy_id=self.policy_id)

    def test_identity_delete_policy_association_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'delete_policy_association_for_endpoint',
            expected_status=204,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_delete_policy_association_for_service(self):
        self.admin_policies_client.update_policy_association_for_service(
            policy_id=self.policy_id, service_id=self.service_id)
        self.do_request(
            'delete_policy_association_for_service',
            expected_status=204,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_delete_policy_association_for_region_and_service(self):
        update_fn = getattr(
            self.admin_policies_client,
            'update_policy_association_for_region_and_service'
        )
        update_fn(policy_id=self.policy_id,
                  service_id=self.service_id,
                  region_id=self.region_id)
        self.do_request(
            'delete_policy_association_for_region_and_service',
            expected_status=204,
            policy_id=self.policy_id,
            service_id=self.service_id,
            region_id=self.region_id)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_policy_association_for_endpoint(self):
        self.do_request(
            'update_policy_association_for_endpoint',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_create_policy_association_for_service(self):
        self.do_request(
            'update_policy_association_for_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_create_policy_association_for_region_and_service(self):
        self.do_request(
            'update_policy_association_for_region_and_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, service_id=self.service_id,
            region_id=self.region_id)

    def test_identity_delete_policy_association_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'delete_policy_association_for_endpoint',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_delete_policy_association_for_service(self):
        self.admin_policies_client.update_policy_association_for_service(
            policy_id=self.policy_id, service_id=self.service_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_service,
            policy_id=self.policy_id, service_id=self.service_id)
        self.do_request(
            'delete_policy_association_for_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_delete_policy_association_for_region_and_service(self):
        update_fn = getattr(
            self.admin_policies_client,
            'update_policy_association_for_region_and_service'
        )
        update_fn(policy_id=self.policy_id,
                  service_id=self.service_id,
                  region_id=self.region_id)
        delete_fn = getattr(
            self.admin_policies_client,
            'delete_policy_association_for_region_and_service'
        )
        self.addCleanup(delete_fn,
                        policy_id=self.policy_id,
                        service_id=self.service_id,
                        region_id=self.region_id)
        self.do_request(
            'delete_policy_association_for_region_and_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id,
            service_id=self.service_id,
            region_id=self.region_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_check_policy_association_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'show_policy_association_for_endpoint',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)

    def test_identity_check_policy_association_for_service(self):
        self.admin_policies_client.update_policy_association_for_service(
            policy_id=self.policy_id, service_id=self.service_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_service,
            policy_id=self.policy_id, service_id=self.service_id)
        self.do_request(
            'show_policy_association_for_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id, service_id=self.service_id)

    def test_identity_check_policy_association_for_region_and_service(self):
        update_fn = getattr(
            self.admin_policies_client,
            'update_policy_association_for_region_and_service'
        )
        update_fn(policy_id=self.policy_id,
                  service_id=self.service_id,
                  region_id=self.region_id)
        delete_fn = getattr(
            self.admin_policies_client,
            'delete_policy_association_for_region_and_service'
        )
        self.addCleanup(delete_fn,
                        policy_id=self.policy_id,
                        service_id=self.service_id,
                        region_id=self.region_id)
        self.do_request(
            'show_policy_association_for_region_and_service',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id,
            service_id=self.service_id,
            region_id=self.region_id)

    def test_identity_get_policy_for_endpoint(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'show_policy_for_endpoint',
            expected_status=exceptions.Forbidden,
            endpoint_id=self.endpoint_id)

    def test_identity_list_endpoints_for_policy(self):
        self.admin_policies_client.update_policy_association_for_endpoint(
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.addCleanup(
            self.admin_policies_client.delete_policy_association_for_endpoint,
            policy_id=self.policy_id, endpoint_id=self.endpoint_id)
        self.do_request(
            'list_endpoints_for_policy',
            expected_status=exceptions.Forbidden,
            policy_id=self.policy_id)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacEndpointGroupTests(rbac_base.IdentityV3RbacBaseTests,
                                       metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacEndpointGroupTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.eg_client = cls.persona.endpoint_groups_client
        cls.ef_client = cls.persona.endpoint_filter_client
        cls.admin_client = cls.os_system_admin
        cls.admin_eg_client = cls.admin_client.endpoint_groups_client
        cls.admin_ef_client = cls.admin_client.endpoint_filter_client

    def endpoint_group(self):
        return {
            'name': data_utils.rand_name('endpoint_group'),
            'filters': {'interface': 'public'},
        }

    @abc.abstractmethod
    def test_identity_create_endpoint_group(self):
        """Test identity:create_endpoint_group policy.

        This test must check:
          * whether the persona can create an endpoint group
        """
        pass

    @abc.abstractmethod
    def test_identity_get_endpoint_group(self):
        """Test identity:get_endpoint_group policy.

        This test must check:
          * whether the persona can get an endpoint group
          * whether the persona can get an endpoint group that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_endpoint_groups(self):
        """Test identity:list_endpoint_groups policy.

        This test must check:
          * whether the persona can list all endpoint groups
        """
        pass

    @abc.abstractmethod
    def test_identity_update_endpoint_group(self):
        """Test identity:update_endpoint_group policy.

        This test must check:
          * whether the persona can update an endpoint group
          * whether the persona can update an endpoint group that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_endpoint_group(self):
        """Test identity:delete_endpoint_group policy.

        This test must check
          * whether the persona can delete an endpoint group
          * whether the persona can delete an endpoint group that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_projects_associated_with_endpoint_group(self):
        """Test identity:list_projects_associated_with_endpoint_group policy.

        This test must check
          * whether the persona can list projects associated with an endpoint
            group
        """
        pass

    @abc.abstractmethod
    def test_identity_list_endpoints_associated_with_endpoint_group(self):
        """Test identity:list_endpoints_associated_with_endpoint_group

        This test must check
          * whether the persona can list endpoints associated with an endpoint
            group
        """
        pass

    @abc.abstractmethod
    def test_identity_get_endpoint_group_in_project(self):
        """Test identity:get_endpoint_group_in_project

        This test must check
          * whether the persona can check if an endpoint group is associated
            with a project
        """
        pass

    @abc.abstractmethod
    def test_identity_list_endpoint_groups_for_project(self):
        """Test identity:list_endpoint_groups_for_project

        This test must check
          * whether the persona can list endpoint groups associated with a
            project
        """
        pass

    @abc.abstractmethod
    def test_identity_add_endpoint_group_to_project(self):
        """Test identity:add_endpoint_group_to_project

        This test must check
          * whether the persona can allow a project to access an endpoint group
        """
        pass

    @abc.abstractmethod
    def test_identity_remove_endpoint_group_from_project(self):
        """Test identity:remove_endpoint_group_from_project

        This test must check
          * whether the persona can remove an endpoint group from a project
        """
        pass


class SystemAdminTests(
    IdentityV3RbacEndpointGroupTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_endpoint_group(self):
        eg = self.do_request('create_endpoint_group', expected_status=201,
                             client=self.eg_client,
                             **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)

    def test_identity_get_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        # user can get an endpoint group
        self.do_request('show_endpoint_group', client=self.eg_client,
                        endpoint_group_id=eg)
        # nonexistent endpoint group gives a 404
        self.do_request('show_endpoint_group',
                        expected_status=exceptions.NotFound,
                        client=self.eg_client,
                        endpoint_group_id=data_utils.rand_uuid_hex())

    def test_identity_list_endpoint_groups(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        resp = self.do_request('list_endpoint_groups',
                               client=self.eg_client)['endpoint_groups']
        self.assertIn(eg, [e['id'] for e in resp])

    def test_identity_update_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        # user can update an endpoint group
        self.do_request('update_endpoint_group', client=self.eg_client,
                        endpoint_group_id=eg,
                        description=data_utils.arbitrary_string())
        # nonexistent endpoint group gives a 404
        self.do_request('update_endpoint_group', client=self.eg_client,
                        expected_status=exceptions.NotFound,
                        endpoint_group_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        # user can delete an endpoint group
        self.do_request('delete_endpoint_group',
                        expected_status=204,
                        client=self.eg_client,
                        endpoint_group_id=eg)
        # nonexistent endpoint group gives a 404
        self.do_request('delete_endpoint_group',
                        expected_status=exceptions.NotFound,
                        client=self.eg_client,
                        endpoint_group_id=data_utils.rand_uuid_hex())

    def test_identity_list_projects_associated_with_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        resp = self.do_request('list_projects_for_endpoint_group',
                               client=self.ef_client,
                               endpoint_group_id=eg)['projects']
        self.assertIn(project, [p['id'] for p in resp])

    def test_identity_list_endpoints_associated_with_endpoint_group(self):
        service = self.admin_client.identity_services_v3_client.create_service(
            type=data_utils.rand_name('service'))['service']['id']
        self.addCleanup(
            self.admin_client.identity_services_v3_client.delete_service,
            service)
        endpoint = self.admin_client.endpoints_v3_client.create_endpoint(
            interface='public',
            url='http://localhost/foo',
            service_id=service)['endpoint']['id']
        self.addCleanup(self.admin_client.endpoints_v3_client.delete_endpoint,
                        endpoint)
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        resp = self.do_request('list_endpoints_for_endpoint_group',
                               client=self.ef_client,
                               endpoint_group_id=eg)['endpoints']
        self.assertIn(endpoint, [e['id'] for e in resp])

    def test_identity_get_endpoint_group_in_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        self.do_request('show_endpoint_group_for_project',
                        client=self.ef_client,
                        endpoint_group_id=eg,
                        project_id=project)

    def test_identity_list_endpoint_groups_for_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        resp = self.do_request('list_endpoint_groups_for_project',
                               client=self.ef_client,
                               project_id=project)
        self.assertIn(eg, [e['id'] for e in resp['endpoint_groups']])

    def test_identity_add_endpoint_group_to_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.do_request('add_endpoint_group_to_project',
                        client=self.ef_client,
                        expected_status=204,
                        endpoint_group_id=eg,
                        project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)

    def test_identity_remove_endpoint_group_from_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.do_request('delete_endpoint_group_from_project',
                        client=self.ef_client,
                        expected_status=204,
                        endpoint_group_id=eg, project_id=project)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_endpoint_group(self):
        self.do_request('create_endpoint_group',
                        expected_status=exceptions.Forbidden,
                        client=self.eg_client,
                        **self.endpoint_group())

    def test_identity_update_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        # user can update an endpoint group
        self.do_request('update_endpoint_group', client=self.eg_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg,
                        description=data_utils.arbitrary_string())
        # nonexistent endpoint group gives a 403
        self.do_request('update_endpoint_group', client=self.eg_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        # user cannot delete an endpoint group
        self.do_request('delete_endpoint_group',
                        expected_status=exceptions.Forbidden,
                        client=self.eg_client,
                        endpoint_group_id=eg)
        # nonexistent endpoint group gives a 403
        self.do_request('delete_endpoint_group',
                        expected_status=exceptions.Forbidden,
                        client=self.eg_client,
                        endpoint_group_id=data_utils.rand_uuid_hex())

    def test_identity_add_endpoint_group_to_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.do_request('add_endpoint_group_to_project',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg,
                        project_id=project)

    def test_identity_remove_endpoint_group_from_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        self.do_request('delete_endpoint_group_from_project',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg, project_id=project)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        # user cannot get an endpoint group
        self.do_request('show_endpoint_group', client=self.eg_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg)
        # nonexistent endpoint group gives a 403
        self.do_request('show_endpoint_group',
                        expected_status=exceptions.Forbidden,
                        client=self.eg_client,
                        endpoint_group_id=data_utils.rand_uuid_hex())

    def test_identity_list_endpoint_groups(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        self.do_request('list_endpoint_groups',
                        expected_status=exceptions.Forbidden,
                        client=self.eg_client)

    def test_identity_list_projects_associated_with_endpoint_group(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        self.do_request('list_projects_for_endpoint_group',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg)

    def test_identity_list_endpoints_associated_with_endpoint_group(self):
        service = self.admin_client.identity_services_v3_client.create_service(
            type=data_utils.rand_name('service'))['service']['id']
        self.addCleanup(
            self.admin_client.identity_services_v3_client.delete_service,
            service)
        endpoint = self.admin_client.endpoints_v3_client.create_endpoint(
            interface='public',
            url='http://localhost/foo',
            service_id=service)['endpoint']['id']
        self.addCleanup(self.admin_client.endpoints_v3_client.delete_endpoint,
                        endpoint)
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        self.do_request('list_endpoints_for_endpoint_group',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg)

    def test_identity_get_endpoint_group_in_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        self.do_request('show_endpoint_group_for_project',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        endpoint_group_id=eg,
                        project_id=project)

    def test_identity_list_endpoint_groups_for_project(self):
        eg = self.admin_eg_client.create_endpoint_group(
            **self.endpoint_group())['endpoint_group']['id']
        self.addCleanup(self.admin_eg_client.delete_endpoint_group, eg)
        project = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name('project'))['project']['id']
        self.addCleanup(self.admin_client.projects_client.delete_project,
                        project)
        self.admin_ef_client.add_endpoint_group_to_project(
            endpoint_group_id=eg, project_id=project)
        self.addCleanup(
            self.admin_ef_client.delete_endpoint_group_from_project,
            endpoint_group_id=eg, project_id=project)
        self.do_request('list_endpoint_groups_for_project',
                        client=self.ef_client,
                        expected_status=exceptions.Forbidden,
                        project_id=project)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest import clients
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacTokenTest(rbac_base.IdentityV3RbacBaseTests,
                              metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacTokenTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.identity_v3_client

    @classmethod
    def resource_setup(cls):
        admin_client = cls.os_system_admin
        cls.user = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        cls.user_id = admin_client.users_v3_client.create_user(
            **cls.user)['user']['id']
        cls.addClassResourceCleanup(
            admin_client.users_v3_client.delete_user, user_id=cls.user_id)
        cls.project_id = admin_client.projects_client.create_project(
            name=data_utils.rand_name()
        )['project']['id']
        cls.addClassResourceCleanup(
            admin_client.projects_client.delete_project,
            project_id=cls.project_id)
        cls.domain_id = admin_client.domains_client.create_domain(
            name=data_utils.rand_name()
        )['domain']['id']
        cls.addClassResourceCleanup(
            admin_client.domains_client.delete_domain,
            domain_id=cls.domain_id)
        cls.addClassResourceCleanup(
            admin_client.domains_client.update_domain,
            domain_id=cls.domain_id, enabled=False)
        role_id = admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        cls.addClassResourceCleanup(
            admin_client.roles_v3_client.delete_role,
            role_id=role_id)
        admin_client.roles_v3_client.create_user_role_on_project(
            project_id=cls.project_id,
            user_id=cls.user_id,
            role_id=role_id
        )
        admin_client.roles_v3_client.create_user_role_on_domain(
            domain_id=cls.domain_id,
            user_id=cls.user_id,
            role_id=role_id
        )
        admin_client.roles_v3_client.create_user_role_on_system(
            user_id=cls.user_id,
            role_id=role_id
        )

    def setUp(self):
        super(IdentityV3RbacTokenTest, self).setUp()
        own_creds = auth.KeystoneV3Credentials(**self.own_keystone_creds)
        own_creds = clients.get_auth_provider(own_creds).fill_credentials()
        self.own_token = clients.Manager(
            credentials=own_creds).identity_v3_client.token
        project_creds = auth.KeystoneV3Credentials(
            user_id=self.user_id,
            password=self.user['password'],
            project_id=self.project_id)
        project_creds = clients.get_auth_provider(
            project_creds).fill_credentials()
        self.project_token = clients.Manager(
            credentials=project_creds).identity_v3_client.token
        domain_creds = auth.KeystoneV3Credentials(
            user_id=self.user_id,
            password=self.user['password'],
            domain_id=self.domain_id)
        domain_creds = clients.get_auth_provider(
            domain_creds).fill_credentials()
        self.domain_token = clients.Manager(
            credentials=domain_creds).identity_v3_client.token
        system_creds = auth.KeystoneV3Credentials(
            user_id=self.user_id,
            password=self.user['password'],
            system='all')
        system_creds = clients.get_auth_provider(
            system_creds).fill_credentials()
        self.system_token = clients.Manager(
            credentials=system_creds).identity_v3_client.token

    @abc.abstractmethod
    def test_identity_check_token(self):
        """Test identity:check_token policy.

        This test must check:
          * whether the persona can check their own token in their current
            scope
          * whether the persona can check a system-scoped token for a different
            user
          * whether the persona can check a domain-scoped token for a different
            user
          * whether the persona can check a project-scoped token for a
            different user
        """
        pass

    @abc.abstractmethod
    def test_identity_validate_token(self):
        """Test identity:validate_token policy.

        This test must validate:
          * whether the persona can validate their own token in their current
            scope
          * whether the persona can validate a system-scoped token for a
            different user
          * whether the persona can validate a domain-scoped token for a
            different user
          * whether the persona can validate a project-scoped token for a
            different user
        """
        pass

    @abc.abstractmethod
    def test_identity_revoke_token(self):
        """Test identity:revoke_token policy.

        This test must revoke:
          * whether the persona can revoke their own token in their current
            scope
          * whether the persona can revoke a system-scoped token for a
            different user
          * whether the persona can revoke a domain-scoped token for a
            different user
          * whether the persona can revoke a project-scoped token for a
            different user
        """
        pass


class SystemAdminTests(IdentityV3RbacTokenTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def setUp(self):
        self.own_keystone_creds = {
            'user_id': self.persona.credentials.user_id,
            'password': self.persona.credentials.password,
            'system': 'all'
        }
        super(SystemAdminTests, self).setUp()

    def test_identity_check_token(self):
        # user can check own token
        self.do_request('check_token_existence', resp_token=self.own_token)
        # user can check other system user's token
        self.do_request('check_token_existence', resp_token=self.system_token)
        # user can check domain user's token
        self.do_request('check_token_existence', resp_token=self.domain_token)
        # user can check project user's token
        self.do_request('check_token_existence', resp_token=self.project_token)

    def test_identity_validate_token(self):
        # user can validate own token
        self.do_request('show_token', resp_token=self.own_token)
        # user can validate other system user's token
        self.do_request('show_token', resp_token=self.system_token)
        # user can validate domain user's token
        self.do_request('show_token', resp_token=self.domain_token)
        # user can validate project user's token
        self.do_request('show_token', resp_token=self.project_token)

    def test_identity_revoke_token(self):
        # user can revoke own token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.own_token)
        # user can revoke other system user's token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.system_token)
        # user can revoke domain user's token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.domain_token)
        # user can revoke project user's token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.project_token)


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_revoke_token(self):
        # user can revoke own token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.own_token)
        # user cannot revoke other system user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.system_token)
        # user cannot revoke domain user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.domain_token)
        # user cannot revoke project user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.project_token)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemAdminTests):

    credentials = ['domain_admin', 'system_admin']

    def setUp(self):
        self.own_keystone_creds = {
            'user_id': self.persona.credentials.user_id,
            'password': self.persona.credentials.password,
            'domain_id': self.persona.credentials.domain_id
        }
        # call base setUp directly to ensure we don't use system creds
        super(SystemAdminTests, self).setUp()


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_check_token(self):
        # user can check own token
        self.do_request('check_token_existence', resp_token=self.own_token)
        # user cannot check other system user's token
        self.do_request('check_token_existence',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.system_token)
        # user cannot check domain user's token
        self.do_request('check_token_existence',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.domain_token)
        # user cannot check project user's token
        self.do_request('check_token_existence',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.project_token)

    def test_identity_validate_token(self):
        # user can validate own token
        self.do_request('show_token', resp_token=self.own_token)
        # user cannot validate other system user's token
        self.do_request('show_token',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.system_token)
        # user cannot validate domain user's token
        self.do_request('show_token',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.domain_token)
        # user cannot validate project user's token
        self.do_request('show_token',
                        expected_status=exceptions.Forbidden,
                        resp_token=self.project_token)

    def test_identity_revoke_token(self):
        # user can revoke own token
        self.do_request('delete_token', expected_status=204,
                        resp_token=self.own_token)
        # user cannot revoke other system user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.system_token)
        # user cannot revoke domain user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.domain_token)
        # user cannot revoke project user's token
        self.do_request('delete_token', expected_status=exceptions.Forbidden,
                        resp_token=self.project_token)


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(DomainAdminTests):

    credentials = ['project_admin', 'system_admin']

    def setUp(self):
        self.own_keystone_creds = {
            'user_id': self.persona.credentials.user_id,
            'password': self.persona.credentials.password,
            'project_id': self.persona.credentials.project_id
        }
        # call base setUp directly to ensure we don't use system creds
        super(SystemAdminTests, self).setUp()


class ProjectManagerTests(DomainMemberTests):

    credentials = ['project_manager', 'system_admin']

    def setUp(self):
        self.own_keystone_creds = {
            'user_id': self.persona.credentials.user_id,
            'password': self.persona.credentials.password,
            'project_id': self.persona.credentials.project_id
        }
        # call base setUp directly to ensure we don't use system creds
        super(SystemAdminTests, self).setUp()


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']

    def setUp(self):
        self.own_keystone_creds = {
            'user_id': self.persona.credentials.user_id,
            'password': self.persona.credentials.password,
            'project_id': self.persona.credentials.project_id
        }
        # call base setUp directly to ensure we don't use system creds
        super(SystemAdminTests, self).setUp()


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import http.client
from tempest.lib.common import rest_client

from keystone_tempest_plugin.services.identity import clients


class RegisteredLimitsClient(clients.Identity):

    subpath_prefix = 'registered_limits'
    collection_url = subpath_prefix
    entity_url = subpath_prefix + '/%s'

    def create_registered_limits(self, payload):
        """Create a list of registered limits.

        :param body: A list of registered limits objects.
        """
        post_body = json.dumps({'registered_limits': payload})
        resp, body = super(RegisteredLimitsClient, self).post(
            self.collection_url, post_body)
        self.expected_success(http.client.CREATED, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def list_registered_limits(self, **kwargs):
        """List registered limits.

        :param kwargs: Filter by service_id, region_id, or resource_name
        """
        resp, body = super(RegisteredLimitsClient, self).get(
            self.collection_url, **kwargs)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def update_registered_limit(self, registered_limit_id, registered_limit):
        """Update a registered limit.

        :param registered_limit_id: ID of registered limit to update
        :param registered_limit: new registered limit object
        """
        patch_body = json.dumps({'registered_limit': registered_limit})
        resp, body = super(RegisteredLimitsClient, self).patch(
            self.entity_url % registered_limit_id, patch_body)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def show_registered_limit(self, registered_limit_id):
        """Get a registered limit.

        :param registered_limit_id: ID of registered limit to show
        """
        resp, body = super(RegisteredLimitsClient, self).get(
            self.entity_url % registered_limit_id)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def delete_registered_limit(self, registered_limit_id):
        """Delete a registered limit.

        :param registered_limit_id: ID of registered limit to delete.
        """
        resp, body = super(RegisteredLimitsClient, self).delete(
            self.entity_url % registered_limit_id)
        self.expected_success(http.client.NO_CONTENT, resp.status)
        return rest_client.ResponseBody(resp, body)


class LimitsClient(clients.Identity):

    subpath_prefix = 'limits'
    collection_url = subpath_prefix
    entity_url = subpath_prefix + '/%s'

    def limits_model(self):
        """Get limits model from server."""
        url = self.entity_url % 'model'
        resp, body = super(LimitsClient, self).get(url)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def create_limits(self, payload):
        """Create a list of project limits.

        :param body: A list of project limits objects.
        """
        post_body = json.dumps({'limits': payload})
        resp, body = super(LimitsClient, self).post(
            self.collection_url, post_body)
        self.expected_success(http.client.CREATED, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def list_limits(self, **kwargs):
        """List project limits.

        :param kwargs: Filter by service_id, region_id, resource_name,
            or project/domain ID
        """
        resp, body = super(LimitsClient, self).get(
            self.collection_url, **kwargs)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def update_limit(self, limit_id, limit):
        """Update a project limit.

        :param limit_id: ID of project limit to update
        :param limit: new project limit object
        """
        patch_body = json.dumps({'limit': limit})
        resp, body = super(LimitsClient, self).patch(
            self.entity_url % limit_id, patch_body)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def show_limit(self, limit_id):
        """Get a project limit.

        :param limit_id: ID of project limit to show
        """
        resp, body = super(LimitsClient, self).get(self.entity_url % limit_id)
        self.expected_success(http.client.OK, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def delete_limit(self, limit_id):
        """Delete a project limit.

        :param limit_id: ID of project limit to delete.
        """
        resp, body = super(LimitsClient, self).delete(
            self.entity_url % limit_id)
        self.expected_success(http.client.NO_CONTENT, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin import clients
from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacMappingTests(rbac_base.IdentityV3RbacBaseTests,
                                 metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacMappingTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.keystone_manager = clients.Manager(cls.persona.credentials)
        persona_mgr = clients.Manager(cls.persona.credentials)
        cls.client = persona_mgr.mapping_rules_client
        admin_client = cls.os_system_admin
        admin_mgr = clients.Manager(admin_client.credentials)
        cls.admin_mapping_client = admin_mgr.mapping_rules_client

    @abc.abstractmethod
    def test_identity_create_mapping(self):
        """Test identity:create_mapping policy.

        This test must check:
          * whether the persona can create a mapping
        """
        pass

    @abc.abstractmethod
    def test_identity_get_mapping(self):
        """Test identity:get_mapping policy.

        This test must check:
          * whether the persona can get a mapping
          * whether the persona can get a mapping that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_mappings(self):
        """Test identity:list_mappings policy.

        This test must check:
          * whether the persona can list all mappings
        """
        pass

    @abc.abstractmethod
    def test_identity_update_mapping(self):
        """Test identity:update_mapping policy.

        This test must check:
          * whether the persona can update a mapping
          * whether the persona can update a mapping that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_mapping(self):
        """Test identity:delete_mapping policy.

        This test must check
          * whether the persona can delete a mapping
          * whether the persona can delete a mapping that does not
            exist
        """
        pass


_RULES = {
    "rules":
        [{
            "local": [],
            "remote": [{"type": data_utils.rand_name()}]
        }]
}


class SystemAdminTests(IdentityV3RbacMappingTests, base.BaseIdentityTest):
    credentials = ['system_admin']

    def test_identity_create_mapping(self):
        mapping_id = self.do_request(
            'create_mapping_rule', expected_status=201,
            mapping_id=data_utils.rand_name(),
            rules=_RULES
        )['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)

    def test_identity_get_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('show_mapping_rule', mapping_id=mapping_id)
        # user gets a 404 for nonexistent mapping
        self.do_request('show_mapping_rule',
                        expected_status=exceptions.NotFound,
                        mapping_id=data_utils.rand_uuid_hex())

    def test_identity_list_mappings(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        resp = self.do_request('list_mapping_rules')
        self.assertIn(mapping_id, [i['id'] for i in resp['mappings']])

    def test_identity_update_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('update_mapping_rule',
                        mapping_id=mapping_id,
                        rules=_RULES)
        # user gets a 404 for nonexistent mapping
        self.do_request('update_mapping_rule',
                        expected_status=exceptions.NotFound,
                        mapping_id=data_utils.rand_uuid_hex(),
                        rules=_RULES)

    def test_identity_delete_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.do_request('delete_mapping_rule', expected_status=204,
                        mapping_id=mapping_id)
        # user gets a 404 for nonexistent mapping
        self.do_request('delete_mapping_rule',
                        expected_status=exceptions.NotFound,
                        mapping_id=mapping_id)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_mapping(self):
        self.do_request('create_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=data_utils.rand_name(),
                        rules=_RULES)

    def test_identity_update_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('update_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=mapping_id,
                        rules=_RULES)
        # user gets a 403 for nonexistent mapping
        self.do_request('update_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=data_utils.rand_uuid_hex(),
                        rules=_RULES)

    def test_identity_delete_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('delete_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=mapping_id)
        # user gets a 403 for nonexistent mapping
        self.do_request('delete_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=mapping_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_mapping(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('show_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=mapping_id)
        # user gets a 403 for nonexistent mapping
        self.do_request('show_mapping_rule',
                        expected_status=exceptions.Forbidden,
                        mapping_id=data_utils.rand_uuid_hex())

    def test_identity_list_mappings(self):
        mapping_id = self.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(),
            rules=_RULES)['mapping']['id']
        self.addCleanup(self.admin_mapping_client.delete_mapping_rule,
                        mapping_id)
        self.do_request('list_mapping_rules',
                        expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

identity_feature_options = [
    cfg.BoolOpt('federation',
                default=False,
                help='Does the environment support the Federated Identity '
                     'feature?'),
    cfg.BoolOpt('external_idp',
                default=True,
                help='Whether to test federated scenarios against an external '
                     'identity provider. If disabled, only '
                     'Keystone-to-Keystone tests will be enabled.'),
    cfg.BoolOpt('enforce_scope',
                default=False,
                help='Does the keystone service enforce scope and use '
                     'scope-aware policies?'),
]

fed_scenario_group = cfg.OptGroup(name='fed_scenario',
                                  title='Federation Scenario Tests Options')

FedScenarioGroup = [
    # Identity Provider
    cfg.StrOpt('idp_id',
               help='The Identity Provider ID'),
    cfg.ListOpt('idp_remote_ids',
                default=[],
                help='The Identity Provider remote IDs list'),
    cfg.StrOpt('idp_username',
               help='Username used to login in the Identity Provider'),
    cfg.StrOpt('idp_password',
               help='Password used to login in the Identity Provider',
               secret=True),
    cfg.StrOpt('idp_ecp_url',
               help='Identity Provider SAML2/ECP URL'),
    cfg.StrOpt('idp_oidc_url',
               help='Identity Provider OIDC URL'),

    # client id (oidc)
    cfg.StrOpt('idp_client_id',
               help='Identity Provider Client ID'),
    cfg.StrOpt('idp_client_secret',
               help='Identity Provider Client Secret'),

    # existing user (oidc)
    cfg.StrOpt('idp_test_user_name',
               help='Identity Provider Test User Name'),
    cfg.StrOpt('idp_test_user_password',
               help='Identity Provider Test User Password',
               secret=True),

    # Mapping rules
    cfg.StrOpt('mapping_remote_type',
               help='The assertion attribute to be used in the remote rules'),
    cfg.StrOpt('mapping_user_name',
               default='{0}',
               help='The username to be used in the local rules.'),
    cfg.StrOpt('mapping_group_name',
               default='federated_users',
               help='The group name to be used in the local rules. The group '
                    'must have at least one assignment in one project.'),
    cfg.StrOpt('mapping_group_domain_name',
               default='federated_domain',
               help='The domain name where the "mapping_group_name" is '
                    'created.'),
    # TODO(cmurphy): remove this option and set to true when all supported
    # branches support the openstack_groups feature
    cfg.BoolOpt('enable_k2k_groups_mapping',
                default=False,
                help='Whether to test support for openstack_groups in the K2K '
                     'SAML assertion (lp#1687593)'),

    # Protocol
    cfg.StrOpt('protocol_id',
               default='mapped',
               help='The Protocol ID'),
]



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin import clients
from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacProtocolTests(rbac_base.IdentityV3RbacBaseTests,
                                  metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacProtocolTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.keystone_manager = clients.Manager(cls.persona.credentials)
        persona_mgr = clients.Manager(cls.persona.credentials)
        cls.client = persona_mgr.identity_providers_client
        admin_client = cls.os_system_admin
        admin_mgr = clients.Manager(admin_client.credentials)
        cls.admin_idp_client = admin_mgr.identity_providers_client
        cls.admin_mapping_client = admin_mgr.mapping_rules_client

    @classmethod
    def setUpClass(cls):
        super(IdentityV3RbacProtocolTests, cls).setUpClass()
        cls.idp_id = cls.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']['id']
        cls.addClassResourceCleanup(
            cls.admin_idp_client.delete_identity_provider, cls.idp_id)
        rules = {
            "rules":
                [{
                    "local": [],
                    "remote": [{"type": data_utils.rand_name()}]
                }]
        }
        cls.mapping_id = cls.admin_mapping_client.create_mapping_rule(
            mapping_id=data_utils.rand_name(), rules=rules)['mapping']['id']
        cls.addClassResourceCleanup(
            cls.admin_mapping_client.delete_mapping_rule,
            mapping_id=cls.mapping_id)

    @abc.abstractmethod
    def test_identity_create_protocol(self):
        """Test identity:create_protocol policy.

        This test must check:
          * whether the persona can create a protocol
        """
        pass

    @abc.abstractmethod
    def test_identity_get_protocol(self):
        """Test identity:get_protocol policy.

        This test must check:
          * whether the persona can get a protocol
          * whether the persona can get a protocol that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_protocols(self):
        """Test identity:list_protocols policy.

        This test must check:
          * whether the persona can list all identity providers
        """
        pass

    @abc.abstractmethod
    def test_identity_update_protocol(self):
        """Test identity:update_protocol policy.

        This test must check:
          * whether the persona can update a protocol
          * whether the persona can update a protocol that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_protocol(self):
        """Test identity:delete_protocol policy.

        This test must check
          * whether the persona can delete a protocol
          * whether the persona can delete a protocol that does not
            exist
        """
        pass


class SystemAdminTests(IdentityV3RbacProtocolTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_protocol(self):
        protocol_id = self.do_request(
            'add_protocol_and_mapping', expected_status=201,
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id
        )['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)

    def test_identity_get_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('get_protocol_and_mapping',
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        # user gets a 404 for nonexistent idp
        self.do_request('get_protocol_and_mapping',
                        expected_status=exceptions.NotFound,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex())

    def test_identity_list_protocols(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        resp = self.do_request('list_protocols_and_mappings',
                               idp_id=self.idp_id)
        self.assertIn(protocol_id, [p['id'] for p in resp['protocols']])

    def test_identity_update_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('update_protocol_mapping',
                        idp_id=self.idp_id,
                        protocol_id=protocol_id,
                        mapping_id=self.mapping_id)
        # user gets a 404 for nonexistent protocol
        self.do_request('update_protocol_mapping',
                        expected_status=exceptions.NotFound,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex(),
                        mapping_id=self.mapping_id)

    def test_identity_delete_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.do_request('delete_protocol_and_mapping', expected_status=204,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        # user gets a 404 for nonexistent idp
        self.do_request('delete_protocol_and_mapping',
                        expected_status=exceptions.NotFound,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_protocol(self):
        self.do_request('add_protocol_and_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_name(),
                        mapping_id=self.mapping_id)

    def test_identity_update_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('update_protocol_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id,
                        mapping_id=self.mapping_id)
        # user gets a 403 for nonexistent protocol
        self.do_request('update_protocol_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex(),
                        mapping_id=self.mapping_id)

    def test_identity_delete_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('delete_protocol_and_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        # user gets a 403 for nonexistent protocol
        self.do_request('delete_protocol_and_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_protocol(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('get_protocol_and_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        # user gets a 403 for nonexistent idp
        self.do_request('get_protocol_and_mapping',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id,
                        protocol_id=data_utils.rand_uuid_hex())

    def test_identity_list_protocols(self):
        protocol_id = self.admin_idp_client.add_protocol_and_mapping(
            idp_id=self.idp_id,
            protocol_id=data_utils.rand_name(),
            mapping_id=self.mapping_id)['protocol']['id']
        self.addCleanup(self.admin_idp_client.delete_protocol_and_mapping,
                        idp_id=self.idp_id,
                        protocol_id=protocol_id)
        self.do_request('list_protocols_and_mappings',
                        expected_status=exceptions.Forbidden,
                        idp_id=self.idp_id)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2017 AT&T Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from keystone_tempest_plugin.tests import base


class OAUTH1TokensTest(base.BaseIdentityTest):

    def _create_consumer(self):
        """Creates a consumer with a random description."""
        description = data_utils.rand_name('test_create_consumer')
        consumer = self.consumers_client.create_consumer(
            description)['consumer']
        # cleans up created consumers after tests
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.consumers_client.delete_consumer,
                        consumer['id'])
        return consumer

    def _create_request_token(self, consumer):
        """Create request token to authorize access for a consumer."""
        project_id = (
            self.oauth_token_client.auth_provider.credentials.project_id)

        request_token = self.oauth_token_client.create_request_token(
            consumer['id'], consumer['secret'], project_id)
        return request_token

    def _authorize_request_token(self, request_token):
        """Authorize request token to provide access to specific roles."""
        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        project_id = (
            self.oauth_token_client.auth_provider.credentials.project_id)
        roles = self.roles_client.list_user_roles_on_project(
            project_id, user_id)
        role_ids = [role['id'] for role in roles['roles']]

        oauth_verifier = self.oauth_token_client.authorize_request_token(
            request_token['oauth_token'], role_ids)
        return oauth_verifier['token']

    def _create_access_token(self, consumer):
        """Create access token for a consumer."""
        request_token = self._create_request_token(consumer)
        oauth_verifier = self._authorize_request_token(request_token)

        access_token = self.oauth_token_client.create_access_token(
            consumer['id'], consumer['secret'],
            request_token['oauth_token'],
            request_token['oauth_token_secret'],
            oauth_verifier['oauth_verifier'])

        # cleans up access tokens after tests
        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.oauth_token_client.revoke_access_token,
                        user_id, access_token['oauth_token'])
        return access_token

    @decorators.idempotent_id('23d2fe8d-fc8d-4bef-8aaa-289400732c3f')
    def test_create_and_show_consumer(self):
        """Tests to make sure that a consumer with parameters is made."""
        consumer = self._create_consumer()
        # fetch created consumer from client
        fetched_consumer = self.consumers_client.show_consumer(
            consumer['id'])['consumer']
        # assert that the fetched consumer matches the created one and
        # has all parameters
        for key in ['description', 'id', 'links']:
            self.assertEqual(consumer[key], fetched_consumer[key])

    @decorators.idempotent_id('3820f3d0-9b06-4d15-8f01-c7dd4eea30a2')
    def test_delete_consumer(self):
        """Tests the delete functionality for resource consumer."""
        consumer = self._create_consumer()
        # fetch consumer from client to confirm it exists
        fetched_consumer = self.consumers_client.show_consumer(
            consumer['id'])['consumer']
        self.assertEqual(consumer['id'], fetched_consumer['id'])
        # delete existing consumer
        self.consumers_client.delete_consumer(consumer['id'])
        # check that consumer no longer exists
        self.assertRaises(lib_exc.NotFound,
                          self.consumers_client.show_consumer,
                          consumer['id'])

    @decorators.idempotent_id('5a03fa78-3a03-449b-a04c-ef9de80eb6c4')
    def test_update_consumer(self):
        """Tests the update functionality for resource consumer."""
        # create a new consumer to update
        consumer = self._create_consumer()
        # create new description
        new_description = data_utils.rand_name('test_update_consumer')
        # update consumer
        self.consumers_client.update_consumer(consumer['id'],
                                              new_description)
        # check that the same consumer now has the new description
        updated_consumer = self.consumers_client.show_consumer(
            consumer['id'])['consumer']
        self.assertEqual(new_description, updated_consumer['description'])

    @decorators.idempotent_id('6da689b1-39a0-44ee-9624-445159119c57')
    def test_list_consumers(self):
        """Test for listing consumers."""
        # create two consumers to populate list
        new_consumer_one = self._create_consumer()
        new_consumer_two = self._create_consumer()
        # fetch the list of consumers
        consumer_list = self.consumers_client.list_consumers()['consumers']
        # add fetched consumer ids to a list
        id_list = [consumer['id'] for consumer in consumer_list]
        # check if created consumers are in the list
        self.assertIn(new_consumer_one['id'], id_list)
        self.assertIn(new_consumer_two['id'], id_list)

    @decorators.idempotent_id('a17d60e4-7cb5-4e06-9e16-b044f3ee6d94')
    def test_create_request_token(self):
        """Test to create request token for consumer."""
        consumer = self._create_consumer()
        request_token = self._create_request_token(consumer)
        # check that oauth token id and secret exists
        self.assertIsNotNone(request_token['oauth_token'])
        self.assertIsNotNone(request_token['oauth_token_secret'])

    @decorators.idempotent_id('607aecc4-a623-4566-a3a5-bb0e2a6fc9c5')
    def test_authorize_request_token(self):
        """Test to authorize a request token for specific role."""
        consumer = self._create_consumer()
        request_token = self._create_request_token(consumer)
        oauth_verifier = self._authorize_request_token(request_token)
        # check that oauth verifier exists
        self.assertIsNotNone(oauth_verifier['oauth_verifier'])

    @decorators.idempotent_id('7d488fc9-342c-4c12-b6b8-b158e2183925')
    def test_create_access_token(self):
        """Test to create access token for consumer."""
        consumer = self._create_consumer()
        access_token = self._create_access_token(consumer)

        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        project_id = (
            self.oauth_token_client.auth_provider.credentials.project_id)
        fetched_access_token = self.oauth_token_client.get_access_token(
            user_id, access_token['oauth_token'])['access_token']
        # check that access token details matches
        self.assertEqual(access_token['oauth_token'],
                         fetched_access_token['id'])
        self.assertEqual(consumer['id'], fetched_access_token['consumer_id'])
        self.assertEqual(access_token['oauth_expires_at'],
                         fetched_access_token['expires_at'])
        self.assertEqual(project_id, fetched_access_token['project_id'])
        self.assertEqual(user_id, fetched_access_token['authorizing_user_id'])

    @decorators.idempotent_id('1b802896-91a0-4cbb-a8b9-860c7087fad8')
    def test_revoke_access_token(self):
        """Test to delete the access token provided for consumer."""
        consumer = self._create_consumer()
        access_token = self._create_access_token(consumer)

        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        # delete existing access token
        self.oauth_token_client.revoke_access_token(
            user_id, access_token['oauth_token'])
        # check that access token no longer exist
        self.assertRaises(lib_exc.NotFound,
                          self.oauth_token_client.get_access_token,
                          user_id,
                          access_token['oauth_token'])

    @decorators.idempotent_id('5929055d-7c0f-4661-a9a5-15c4b95082dc')
    def test_list_access_tokens(self):
        """Test to list access tokens provided to consumer."""
        # create two consumers and access token for each
        new_consumer_one = self._create_consumer()
        new_consumer_two = self._create_consumer()
        access_token_one = self._create_access_token(new_consumer_one)
        access_token_two = self._create_access_token(new_consumer_two)

        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        # fetch the list of access tokens
        access_token_list = self.oauth_token_client.list_access_tokens(
            user_id)['access_tokens']
        # add fetch access token ids to a list
        id_list = [access_token['id'] for access_token in access_token_list]
        # check if created access tokens are in the list
        self.assertIn(access_token_one['oauth_token'], id_list)
        self.assertIn(access_token_two['oauth_token'], id_list)

    @decorators.idempotent_id('0075f413-e249-42e5-9bc9-d6e3aecf6cbc')
    def test_list_roles_for_access_token(self):
        """Test to list roles for an access token."""
        consumer = self._create_consumer()
        access_token = self._create_access_token(consumer)

        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        project_id = (
            self.oauth_token_client.auth_provider.credentials.project_id)
        fetched_roles = self.oauth_token_client.list_access_token_roles(
            user_id, access_token['oauth_token'])['roles']
        fetched_role_ids = [role['id'] for role in fetched_roles]
        roles = self.roles_client.list_user_roles_on_project(
            project_id, user_id)
        role_ids = [role['id'] for role in roles['roles']]

        # check that role ids matches
        self.assertCountEqual(fetched_role_ids, role_ids)

    @decorators.idempotent_id('28aee994-86b1-4596-a652-572f558045e7')
    def test_show_role_for_access_token(self):
        """Test to show role details for an access token."""
        consumer = self._create_consumer()
        access_token = self._create_access_token(consumer)

        user_id = self.oauth_token_client.auth_provider.credentials.user_id
        project_id = (
            self.oauth_token_client.auth_provider.credentials.project_id)
        roles = self.roles_client.list_user_roles_on_project(
            project_id, user_id)
        fetched_role = self.oauth_token_client.get_access_token_role(
            user_id,
            access_token['oauth_token'],
            roles['roles'][0]['id'])

        # check that role id matches
        self.assertEqual(fetched_role['role']['id'], roles['roles'][0]['id'])



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.common import credentials_factory as common_creds
from tempest import test

from keystone_tempest_plugin import clients


class BaseIdentityTest(test.BaseTestCase):

    # The version of the identity that will be used in the tests.
    identity_version = 'v3'

    # NOTE(rodrigods): for now, all tests are in the admin scope, if
    # necessary, another class can be created to handle non-admin tests.
    credential_type = 'identity_admin'

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityTest, cls).setup_clients()
        credentials = common_creds.get_configured_admin_credentials(
            cls.credential_type, identity_version=cls.identity_version)
        cls.keystone_manager = clients.Manager(credentials)
        cls.auth_client = cls.keystone_manager.auth_client
        cls.idps_client = cls.keystone_manager.identity_providers_client
        cls.mappings_client = cls.keystone_manager.mapping_rules_client
        cls.roles_client = cls.keystone_manager.roles_v3_client
        cls.saml2_client = cls.keystone_manager.saml2_client
        cls.sps_client = cls.keystone_manager.service_providers_client
        cls.tokens_client = cls.keystone_manager.token_v3_client
        cls.consumers_client = cls.keystone_manager.oauth_consumers_client
        cls.oauth_token_client = cls.keystone_manager.oauth_token_client
        cls.registered_limits_client = (
            cls.keystone_manager.registered_limits_client)
        cls.limits_client = cls.keystone_manager.limits_client



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from keystone_tempest_plugin.services.identity import clients


class MappingRulesClient(clients.Federation):

    subpath_suffix = 'mappings'

    def create_mapping_rule(self, mapping_id, rules):
        """Create a mapping rule."""
        put_body = json.dumps({'mapping': rules})
        return self._put(mapping_id, put_body)

    def list_mapping_rules(self):
        """List the mapping rules."""
        return self._get()

    def show_mapping_rule(self, mapping_id):
        """Get a mapping rule."""
        return self._get(mapping_id)

    def delete_mapping_rule(self, mapping_id):
        """Delete a mapping rule."""
        return self._delete(mapping_id)

    def update_mapping_rule(self, mapping_id, rules):
        """Update a mapping rule."""
        patch_body = json.dumps({'mapping': rules})
        return self._patch(mapping_id, patch_body)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystone_tempest_plugin.services.identity.v3 import (
    identity_providers_client)
from keystone_tempest_plugin.services.identity.v3 import (
    mapping_rules_client)
from keystone_tempest_plugin.services.identity.v3 import (
    service_providers_client)
from keystone_tempest_plugin.services.identity.v3 import auth_client
from keystone_tempest_plugin.services.identity.v3 import limits_client
from keystone_tempest_plugin.services.identity.v3 import saml2_client

from tempest import clients


class Manager(clients.Manager):

    def __init__(self, credentials):
        super(Manager, self).__init__(credentials)

        # keystone auth client
        self.auth_client = auth_client.AuthClient(self.auth_provider)

        # federation clients
        self.identity_providers_client = (
            identity_providers_client.IdentityProvidersClient(
                self.auth_provider))
        self.mapping_rules_client = (
            mapping_rules_client.MappingRulesClient(
                self.auth_provider))
        self.saml2_client = saml2_client.Saml2Client()
        self.service_providers_client = (
            service_providers_client.ServiceProvidersClient(
                self.auth_provider))

        # unified limits clients
        self.registered_limits_client = limits_client.RegisteredLimitsClient(
            self.auth_provider)
        self.limits_client = limits_client.LimitsClient(self.auth_provider)



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacServiceTests(rbac_base.IdentityV3RbacBaseTests,
                                 metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacServiceTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.identity_services_v3_client
        admin_client = cls.os_system_admin
        cls.admin_services_client = admin_client.identity_services_v3_client

    def service(self):
        return {
            'name': data_utils.rand_name('service_name'),
            'type': data_utils.rand_name('service_type'),
        }

    @abc.abstractmethod
    def test_identity_create_service(self):
        """Test identity:create_service policy.

        This test must check:
          * whether the persona can create a service
        """
        pass

    @abc.abstractmethod
    def test_identity_get_service(self):
        """Test identity:get_service policy.

        This test must check:
          * whether the persona can get a service
          * whether the persona can get a service that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_services(self):
        """Test identity:list_services policy.

        This test must check:
          * whether the persona can list all services
        """
        pass

    @abc.abstractmethod
    def test_identity_update_service(self):
        """Test identity:update_service policy.

        This test must check:
          * whether the persona can update a service
          * whether the persona can update a service that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_service(self):
        """Test identity:delete_service policy.

        This test must check
          * whether the persona can delete a service
          * whether the persona can delete a service that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacServiceTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_service(self):
        service_id = self.do_request(
            'create_service', expected_status=201,
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)

    def test_identity_get_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        self.do_request('show_service', service_id=service_id)
        # user gets a 404 for nonexistent service
        self.do_request('show_service', expected_status=exceptions.NotFound,
                        service_id=data_utils.rand_uuid_hex())

    def test_identity_list_services(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        resp = self.do_request('list_services')
        self.assertIn(service_id, [e['id'] for e in resp['services']])

    def test_identity_update_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        self.do_request('update_service',
                        service_id=service_id,
                        type=data_utils.rand_name('service_type'))
        # user gets a 404 for nonexistent service
        self.do_request('update_service', expected_status=exceptions.NotFound,
                        service_id=data_utils.rand_uuid_hex(),
                        type=data_utils.rand_name('service_type'))

    def test_identity_delete_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.do_request('delete_service', expected_status=204,
                        service_id=service_id)
        # user gets a 404 for nonexistent service
        self.do_request('delete_service', expected_status=exceptions.NotFound,
                        service_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_service(self):
        self.do_request(
            'create_service', expected_status=exceptions.Forbidden,
            **self.service())

    def test_identity_update_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        self.do_request('update_service',
                        expected_status=exceptions.Forbidden,
                        service_id=service_id,
                        type=data_utils.rand_name('service_type'))
        # user gets a 403 for nonexistent service
        self.do_request('update_service', expected_status=exceptions.Forbidden,
                        service_id=data_utils.rand_uuid_hex(),
                        type=data_utils.rand_name('service_type'))

    def test_identity_delete_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.do_request('delete_service',
                        expected_status=exceptions.Forbidden,
                        service_id=service_id)
        # user gets a 403 for nonexistent service
        self.do_request('delete_service', expected_status=exceptions.Forbidden,
                        service_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_service(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        self.do_request('show_service', expected_status=exceptions.Forbidden,
                        service_id=service_id)
        # user gets a 403 for nonexistent service
        self.do_request('show_service', expected_status=exceptions.Forbidden,
                        service_id=data_utils.rand_uuid_hex())

    def test_identity_list_services(self):
        service_id = self.admin_services_client.create_service(
            **self.service())['service']['id']
        self.addCleanup(
            self.admin_services_client.delete_service,
            service_id=service_id)
        self.do_request('list_services', expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacProjectTagTests(rbac_base.IdentityV3RbacBaseTests,
                                    metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacProjectTagTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.project_tags_client
        cls.admin_client = cls.os_system_admin
        cls.admin_project_tags_client = cls.admin_client.project_tags_client

    @abc.abstractmethod
    def test_identity_create_project_tag(self):
        """Test identity:create_project_tag policy.

        This test must check:
          * whether the persona can create a tag for an arbitrary project
          * whether the persona can create a tag for a project in their own
            domain
          * whether the persona can create a tag for a project in another
            domain
          * whether the persona can create a tag for their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_get_project_tag(self):
        """Test identity:get_project_tag policy.

        This test must check:
          * whether the persona can get a tag for an arbitrary project
          * whether the persona can get a tag for a project in their own domain
          * whether the persona can get a tag for a project in another domain
          * whether the persona can get tag for their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_list_project_tags(self):
        """Test identity:list_project_tags policy.

        This test must check:
          * whether the persona can list tags for an arbitrary project
          * whether the persona can list tags for a project in their own domain
          * whether the persona can list tags for a project in another domain
          * whether the persona can list tags for their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_update_project_tags(self):
        """Test identity:update_project_tags policy.

        This test must check:
          * whether the persona can update all tags for an project
          * whether the persona can update all tags for a project in their own
            domain
          * whether the persona can update all tags for a project in another
            domain
          * whether the persona can update all tags for their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_project_tag(self):
        """Test identity:delete_project_tag policy.

        This test must check
          * whether the persona can delete a single tag for an arbitrary
            project
          * whether the persona can delete a single tag for a project in their
            own domain
          * whether the persona can delete a single tag for a project in
            another domain
          * whether the persona can delete a single tag for their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_project_tags(self):
        """Test identity:delete_project_tag policy.

        This test must check
          * whether the persona can delete all tags for an arbitrary project
          * whether the persona can delete all tags for a project in their own
            domain
          * whether the persona can delete all tags for a project in another
            domain
          * whether the persona can delete all tags for their own project
        """
        pass


class SystemAdminTests(IdentityV3RbacProjectTagTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def setUp(self):
        super(SystemAdminTests, self).setUp()
        self.project_id = self.admin_client.projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(
            self.admin_client.projects_client.delete_project, self.project_id)

    def test_identity_create_project_tag(self):
        self.do_request(
            'update_project_tag', expected_status=201,
            project_id=self.project_id,
            tag=data_utils.rand_uuid_hex()
        )

    def test_identity_get_project_tag(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.project_id, tag=tag)

    def test_identity_list_project_tags(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        resp = self.do_request('list_project_tags', project_id=self.project_id)
        self.assertIn(tag, resp['tags'])

    def test_identity_update_project_tags(self):
        self.do_request('update_all_project_tags',
                        project_id=self.project_id,
                        tags=[data_utils.rand_uuid_hex()])

    def test_identity_delete_project_tag(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        self.do_request('delete_project_tag', expected_status=204,
                        project_id=self.project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        self.do_request('delete_all_project_tags', expected_status=204,
                        project_id=self.project_id)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_project_tag(self):
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.project_id,
            tag=data_utils.rand_uuid_hex()
        )

    def test_identity_update_project_tags(self):
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id,
                        tags=[data_utils.rand_uuid_hex()])

    def test_identity_delete_project_tag(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(IdentityV3RbacProjectTagTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def setUp(self):
        super(DomainAdminTests, self).setUp()
        self.own_domain = self.persona.credentials.domain_id
        self.other_domain = self.admin_client.domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        self.addCleanup(self.admin_client.domains_client.delete_domain,
                        self.other_domain)
        self.addCleanup(self.admin_client.domains_client.update_domain,
                        domain_id=self.other_domain, enabled=False)
        project_client = self.admin_client.projects_client
        self.own_project_id = project_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(
            project_client.delete_project,
            self.own_project_id)
        self.other_project_id = project_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(project_client.delete_project, self.other_project_id)

    def test_identity_create_project_tag(self):
        # user can add tags to project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=201,
            project_id=self.own_project_id,
            tag=tag
        )
        # user can add tags to project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=201,
            project_id=self.other_project_id,
            tag=tag
        )

    def test_identity_get_project_tag(self):
        # user can get tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.own_project_id, tag=tag)
        # user can get tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.other_project_id, tag=tag)

    def test_identity_list_project_tags(self):
        # user can list tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        resp = self.do_request('list_project_tags',
                               project_id=self.own_project_id)
        self.assertIn(tag, resp['tags'])
        # user can list tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        resp = self.do_request('list_project_tags',
                               project_id=self.other_project_id)
        self.assertIn(tag, resp['tags'])

    def test_identity_update_project_tags(self):
        # user can update tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        project_id=self.own_project_id,
                        tags=[tag])
        # user can update tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        project_id=self.other_project_id,
                        tags=[tag])

    def test_identity_delete_project_tag(self):
        # user can delete tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_project_tag', expected_status=204,
                        project_id=self.own_project_id,
                        tag=tag)
        # user can delete tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=204,
                        project_id=self.other_project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        # user can delete tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_all_project_tags', expected_status=204,
                        project_id=self.own_project_id)
        # user can delete tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=204,
                        project_id=self.other_project_id)


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_create_project_tag(self):
        # user can add tags to project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=201,
            project_id=self.own_project_id,
            tag=tag
        )
        # user cannot add tags to project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.other_project_id,
            tag=tag
        )

    def test_identity_get_project_tag(self):
        # user can get tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.own_project_id, tag=tag)
        # user cannot get tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id, tag=tag)

    def test_identity_list_project_tags(self):
        # user can list tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        resp = self.do_request('list_project_tags',
                               project_id=self.own_project_id)
        self.assertIn(tag, resp['tags'])
        # user can list tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('list_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)

    def test_identity_update_project_tags(self):
        # user can update tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        project_id=self.own_project_id,
                        tags=[tag])
        # user cannot update tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tags=[tag])

    def test_identity_delete_project_tag(self):
        # user can delete tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_project_tag', expected_status=204,
                        project_id=self.own_project_id,
                        tag=tag)
        # user cannot delete tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        # user can delete tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_all_project_tags', expected_status=204,
                        project_id=self.own_project_id)
        # user cannot delete tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']

    def test_identity_create_project_tag(self):
        # user cannot add tags to project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.own_project_id,
            tag=tag
        )
        # user cannot add tags to project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.other_project_id,
            tag=tag
        )

    def test_identity_get_project_tag(self):
        # user can get tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.own_project_id, tag=tag)
        # user cannot get tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id, tag=tag)

    def test_identity_list_project_tags(self):
        # user can list tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        resp = self.do_request('list_project_tags',
                               project_id=self.own_project_id)
        self.assertIn(tag, resp['tags'])
        # user cannot list tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('list_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)

    def test_identity_update_project_tags(self):
        # user cannot update tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id,
                        tags=[tag])
        # user cannot update tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tags=[tag])

    def test_identity_delete_project_tag(self):
        # user cannot delete tag for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id,
                        tag=tag)
        # user cannot delete tag for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        # user cannot delete tags for project in own domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id)
        # user cannot delete tags for project in other domain
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(DomainAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainMemberTests):

    credentials = ['project_manager', 'system_admin']

    def setUp(self):
        super().setUp()
        self.own_project_id = self.persona.credentials.project_id
        project_client = self.admin_client.projects_client
        self.other_project_id = project_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(project_client.delete_project, self.other_project_id)

    def test_identity_create_project_tag(self):
        # user cannot add tags to own project
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.own_project_id,
            tag=tag
        )
        # user cannot add tags to arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.do_request(
            'update_project_tag', expected_status=exceptions.Forbidden,
            project_id=self.other_project_id,
            tag=tag
        )

    def test_identity_get_project_tag(self):
        # user can get tag for own project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.addCleanup(self.admin_project_tags_client.delete_project_tag,
                        project_id=self.own_project_id,
                        tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=204,
                        project_id=self.own_project_id, tag=tag)
        # user cannot get tag for arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('check_project_tag_existence',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id, tag=tag)

    def test_identity_list_project_tags(self):
        # user can list tags for own project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.addCleanup(self.admin_project_tags_client.delete_project_tag,
                        project_id=self.own_project_id,
                        tag=tag)
        resp = self.do_request('list_project_tags',
                               project_id=self.own_project_id)
        self.assertIn(tag, resp['tags'])
        # user cannot list tags for arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('list_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)

    def test_identity_update_project_tags(self):
        # user cannot update tags for own project
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id,
                        tags=[tag])
        # user cannot update tags for arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.do_request('update_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tags=[tag])

    def test_identity_delete_project_tag(self):
        # user cannot delete tag for own project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.addCleanup(self.admin_project_tags_client.delete_project_tag,
                        project_id=self.own_project_id,
                        tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id,
                        tag=tag)
        # user cannot delete tag for arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_project_tag',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id,
                        tag=tag)

    def test_identity_delete_project_tags(self):
        # user cannot delete tags for own project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.own_project_id, tag=tag)
        self.addCleanup(self.admin_project_tags_client.delete_project_tag,
                        project_id=self.own_project_id,
                        tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.own_project_id)
        # user cannot delete tags for arbitrary project
        tag = data_utils.rand_uuid_hex()
        self.admin_project_tags_client.update_project_tag(
            project_id=self.other_project_id, tag=tag)
        self.do_request('delete_all_project_tags',
                        expected_status=exceptions.Forbidden,
                        project_id=self.other_project_id)


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacDomainConfigTest(rbac_base.IdentityV3RbacBaseTests,
                                     metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacDomainConfigTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.domain_config_client
        cls.admin_client = cls.os_system_admin
        cls.admin_domain_config_client = cls.admin_client.domain_config_client

    @classmethod
    def resource_setup(cls):
        super(IdentityV3RbacDomainConfigTest, cls).resource_setup()
        cls.domain_id = cls.admin_client.domains_client.create_domain(
            name=data_utils.rand_name('domain'))['domain']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.delete_domain,
            cls.domain_id)
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.update_domain,
            cls.domain_id,
            enabled=False)

    def domain_config(self, **kwargs):
        ref = {
            "identity": {
                "driver": "ldap"
            },
            "ldap": {
                "url": "ldap://myldap.com:389/",
                "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"
            }
        }
        ref.update(kwargs)
        return ref

    @abc.abstractmethod
    def test_identity_create_domain_config(self):
        """Test identity:create_domain_config policy.

        This test must check:
          * whether the persona can create a domain config for a valid domain
        """
        pass

    @abc.abstractmethod
    def test_identity_get_domain_config(self):
        """Test identity:get_domain_config policy.

        This test must check:
          * whether the persona can get a domain config
          * whether the persona can get an option group for a domain config
          * whether the persona can get an option from a group in a domain
            config
          * whether the persona can get a config for an invalid domain
          * whether the persona can get a config that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_get_domain_config_default(self):
        """Test identity:get_domain_config_default policy.

          * whether the persona can get the default config
          * whether the persona can get the default config for an option group
          * whether the persona can get the default value for an option
        """
        pass

    @abc.abstractmethod
    def test_identity_get_security_compliance_domain_config(self):
        """Test identity:get_security_compliance_domain_config policy.

        This test must check:
          * whether the persona can get the security compliance configuration
            for the default domain
          * whether the persona can get an option from the security compliance
            configuration for the default domain
        """
        pass

    @abc.abstractmethod
    def test_identity_update_domain_config(self):
        """Test identity:update_domain_config policy.

        This test must check:
          * whether the persona can update the config for a domain
          * whether the persona can update an option group config for a domain
          * whether the persona can update an option in a domain config
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_domain_config(self):
        """Test identity:delete_domain_config policy.

        This test must check
          * whether the persona can delete a domain config
          * whether the persona can delete an option group within a domain
            config
          * whether the persona can delete an option within a domain config
        """
        pass


class SystemAdminTests(IdentityV3RbacDomainConfigTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_domain_config(self):
        self.do_request(
            'create_domain_config',
            expected_status=201,
            domain_id=self.domain_id,
            **self.domain_config())
        self.addCleanup(
            self.admin_domain_config_client.delete_domain_config,
            self.domain_id)

    def test_identity_get_domain_config(self):
        # should be able to get domain config, group and individual options
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.addCleanup(
            self.admin_domain_config_client.delete_domain_config,
            self.domain_id)
        self.do_request(
            'show_domain_config',
            domain_id=self.domain_id)
        self.do_request(
            'show_domain_group_config',
            domain_id=self.domain_id,
            group='ldap')
        self.do_request(
            'show_domain_group_option_config',
            domain_id=self.domain_id,
            group='ldap',
            option='url')
        # should get Not Found for invalid domain
        self.do_request(
            'show_domain_config',
            expected_status=exceptions.NotFound,
            domain_id=data_utils.rand_uuid_hex())
        # should get Not Found for nonexistent config for valid domain
        domain = self.admin_client.domains_client.create_domain(
            name=data_utils.rand_name('domain'))['domain']['id']
        self.addCleanup(self.admin_client.domains_client.delete_domain, domain)
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            domain, enabled=False)
        self.do_request(
            'show_domain_config',
            expected_status=exceptions.NotFound,
            domain_id=domain)

    def test_identity_get_domain_config_default(self):
        self.do_request('show_default_config_settings')
        self.do_request('show_default_group_config', group='ldap')
        self.do_request(
            'show_default_group_option', group='ldap', option='url')

    def test_identity_get_security_compliance_domain_config(self):
        self.do_request(
            'show_domain_group_config',
            domain_id='default',
            group='security_compliance')
        self.do_request(
            'show_domain_group_option_config',
            domain_id='default',
            group='security_compliance',
            option='password_regex_description')

    def test_identity_update_domain_config(self):
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.addCleanup(
            self.admin_domain_config_client.delete_domain_config,
            self.domain_id)
        self.do_request(
            'update_domain_group_config',
            domain_id=self.domain_id,
            group='ldap',
            ldap={'url': 'ldaps://myldap.com:636/',
                  'user_tree_dn': 'ou=People,dc=my_new_root,dc=org'})
        self.do_request(
            'update_domain_group_option_config',
            domain_id=self.domain_id,
            group='ldap',
            option='user_tree_dn',
            user_tree_dn='ou=Aliens,dc=my_new_root,dc=org')
        # test changing the entire config last
        self.do_request(
            'update_domain_config',
            domain_id=self.domain_id,
            identity={"driver": "sql"})

    def test_identity_delete_domain_config(self):
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.do_request(
            'delete_domain_group_option_config',
            expected_status=204,
            domain_id=self.domain_id,
            group='ldap',
            option='user_tree_dn')
        self.do_request(
            'delete_domain_group_config',
            expected_status=204,
            domain_id=self.domain_id,
            group='ldap')
        self.do_request(
            'delete_domain_config',
            expected_status=204,
            domain_id=self.domain_id)


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_domain_config(self):
        self.do_request(
            'create_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            **self.domain_config())

    def test_identity_update_domain_config(self):
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.addCleanup(
            self.admin_domain_config_client.delete_domain_config,
            self.domain_id)
        self.do_request(
            'update_domain_group_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap',
            ldap={'url': 'ldaps://myldap.com:636/',
                  'user_tree_dn': 'ou=People,dc=my_new_root,dc=org'})
        self.do_request(
            'update_domain_group_option_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap',
            option='user_tree_dn',
            user_tree_dn='ou=Aliens,dc=my_new_root,dc=org')
        # test changing the entire config last
        self.do_request(
            'update_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            identity={"driver": "sql"})

    def test_identity_delete_domain_config(self):
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.do_request(
            'delete_domain_group_option_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap',
            option='user_tree_dn')
        self.do_request(
            'delete_domain_group_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap')
        self.do_request(
            'delete_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_domain_config(self):
        # should not be able to get domain config, group and individual options
        self.admin_domain_config_client.create_domain_config(
            self.domain_id, **self.domain_config())
        self.addCleanup(
            self.admin_domain_config_client.delete_domain_config,
            self.domain_id)
        self.do_request(
            'show_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id)
        self.do_request(
            'show_domain_group_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap')
        self.do_request(
            'show_domain_group_option_config',
            expected_status=exceptions.Forbidden,
            domain_id=self.domain_id,
            group='ldap',
            option='url')
        # should get Forbidden for invalid domain
        self.do_request(
            'show_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=data_utils.rand_uuid_hex())
        # should get Forbidden for nonexistent config for valid domain
        domain = self.admin_client.domains_client.create_domain(
            name=data_utils.rand_name('domain'))['domain']['id']
        self.addCleanup(self.admin_client.domains_client.delete_domain, domain)
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            domain, enabled=False)
        self.do_request(
            'show_domain_config',
            expected_status=exceptions.Forbidden,
            domain_id=domain)

    def test_identity_get_domain_config_default(self):
        self.do_request(
            'show_default_config_settings',
            expected_status=exceptions.Forbidden)
        self.do_request(
            'show_default_group_config',
            expected_status=exceptions.Forbidden, group='ldap')
        self.do_request(
            'show_default_group_option',
            expected_status=exceptions.Forbidden, group='ldap', option='url')


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainAdminTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']
